Imports HelperFunctions

' 
'    Copyright (c) 2012 Microsoft Corporation. All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code. For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see all Code Samples for Windows Phone, visit http://go.microsoft.com/fwlink/?LinkID=219604 
'  
'

Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports Microsoft.Phone.Controls
Imports Windows.Foundation
Imports Windows.Phone.Speech.Recognition
Imports Windows.Phone.Speech.Synthesis
Imports Windows.Phone.Speech.VoiceCommands

' Code behind for the Magic Memo sample code main page. This sample uses simple handlers and helper functions to illustrate the use of 
' speech recognition and synthesis in Window Phone 8. It does not implement MVVM and does not store static strings in resources as a 
' globalized application would.
Namespace MagicMemo
	Partial Public Class MainPage
		Inherits PhoneApplicationPage

		'
		' Code behind for the Magic Memo sample application Main page
		'

		#Region "Speech objects"
		' Speech synthesizer for prompting and confirming
		Private speechOutput As SpeechSynthesizer

		' Speech Recognzier for entering memos using the default dictation grammar
		Private speechInput As SpeechRecognizerUI

		' Speech Recognizer for getting user commands. Uses mainPageCommands to add a list grammar
		Private commandReco As SpeechRecognizerUI

		' List of commands to listen for on the mainpage
		Private mainPageCommands As New List(Of String)
		#End Region

		#Region "Private fields"
		' File in isostorage to store memos
		Private fileName As String = "savedMemos.txt"

		' Uri of the other application page
		Private viewMemosUri As New Uri("/ViewMemos.xaml", UriKind.Relative)

		Private numMemosSaved As Integer = 0
		#End Region

		' Constructor for this page 
		Public Sub New()
			InitializeComponent() ' Autogenerated, do not modify
			InitializeSpeech()
			InitializeCommandGrammar()
		End Sub

		#Region "Event handlers"
		' Loaded event handler for this page
		Private Sub MainPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Handle the case where this page was launched directly by Voice Command
			If Me.NavigationContext.QueryString IsNot Nothing AndAlso Me.NavigationContext.QueryString.ContainsKey("voiceCommandName") Then ' Page was launched by Voice Command
				' Note: This page has only one voice command which is to enter a new memo, so there is no need to 
				' check the value of the command name.

				' Get a new memo using speech recognition
				GetNewMemoByVoice()
			End If
		End Sub

		' Handler for mic tap
		Private Sub mic_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Get a new memo using speech recognition
			GetNewMemoByVoice()
		End Sub

		''' Handler for "Save" button tap
		Private Sub SaveBT_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Append the current memo to the iso-storage file of stored memos
			Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text)
			' Reset textbox text to blank
			Memo_TB.Text = ""

            numMemosSaved = numMemosSaved + 1
			UpdateNumberPhraseList("num", numMemosSaved, "MagicMemoEnu")
		End Sub

		''' Handler for Cancel button tap
		Private Sub Cancel_BT_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Reset textbox text to blank
			Utilities.SetTextBoxTextSafely(Memo_TB, "")
		End Sub

		''' Handler for View Saved Memos button tap
		Private Sub View_BT_Tap(ByVal sender As Object, ByVal e As GestureEventArgs)
			' Navigate to the ViewMemos.xaml page
			Me.NavigationService.Navigate(viewMemosUri)
		End Sub
		#End Region

		#Region "Speech helper functions"
		' Initializes the synthesizer object and two recognizer objects
		Private Sub InitializeSpeech()
			speechOutput = New SpeechSynthesizer

			' Simple dictation recognizer for filling in memo text
			speechInput = New SpeechRecognizerUI

			' Set text to display to the user when recognizing
			speechInput.Settings.ExampleText = "Example: ""Buy roses"""
			speechInput.Settings.ListenText = "Say your memo"

			' Recognizer for getting commands on this page
			commandReco = New SpeechRecognizerUI

			' Set text to display to the user when recognizing
			commandReco.Settings.ExampleText = "Button name, ""Retry"", or ""Quit"""
			commandReco.Settings.ListenText = "Say next command..."

			' Note: Grammars are intialized in the InitCommandGrammars method
		End Sub

		' Initializes the grammar used by the commandReco recognizer object
		Private Sub InitializeCommandGrammar()
			' Recursively add "Content" text of each button to the mainPageCommands string list
			GetButtonContentText(Me.LayoutRoot, mainPageCommands)

			' Add these three commands that don't have buttons to the mainPageCommands string array
			mainPageCommands.Add("Try again")
			mainPageCommands.Add("Quit")
			mainPageCommands.Add("Save and try again")

			' Add grammar consisting of button "Content" text
			commandReco.Recognizer.Grammars.AddGrammarFromList("mainPageCommands", mainPageCommands)
		End Sub

		' Gets a new memo using recognition. Used by mic_Tap handler and some command results
		' in the CommandCompleted callback
		Private async Sub GetNewMemoByVoice()
			Try
				' Prompt the user using speech synthesis
				await speechOutput.SpeakTextAsync("Say your memo")

				' Use the default dictation grammar to get the new memo text
				Dim recoResult = await speechInput.RecognizeWithUIAsync

				If recoResult.ResultStatus = SpeechRecognitionUIStatus.Succeeded Then
					' Show new memo text
					Utilities.SetTextBoxTextSafely(Memo_TB, recoResult.RecognitionResult.Text)

					' Ask the user what to do next and execute the command
					GetCommand()
				Else
					Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show(String.Format("Failure message: {0}", recoResult.ResultStatus.ToString)))
				End If
			Catch ex As Exception
				Const privacyPolicyHResult As Integer = CInt(&H80045509)

				If ex.HResult = privacyPolicyHResult Then ' User has not accepted the speech privacy policy
					Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("The user has not accepted the speech privacy policy."))
				Else
					Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Exception in dictation recognition" & ex.Message))
				End If
			End Try
		End Sub

		' Gets a command using the commandReco recognition object after the user has entered a memo by Speech recognition. 
		' The result is handled in the CommandCompleted callback.
		Private async Sub GetCommand()
			' Prompt the user to say a command
			await speechOutput.SpeakTextAsync("Say a button name, quit, 'save and try again' or 'try again'.")

			Dim operation = commandReco.RecognizeWithUIAsync

			' Use a callback function instead of the await keyword in this case
			operation.Completed = AddressOf CommandCompleted
		End Sub

		' Callback for RecognizeWithUIAsync completion for use with the commandReco recognizer object. It takes the appropriate action based on 
		' the recognition result, which can be any button name or "save and try again", "try again", or "quit". Referenced in the "GetCommand" 
		' helper function
        Private Sub CommandCompleted(ByVal asyncInfo As IAsyncOperation(Of SpeechRecognitionUIResult), ByVal asyncStatusParam As AsyncStatus)
            ' Don't do anything if the recognition hasn't completed.
            If asyncStatusParam <> AsyncStatus.Completed Then
                Exit Sub
            End If

            Try
                ' Get the recognition result object
                Dim result As SpeechRecognitionUIResult = asyncInfo.GetResults

                If result.ResultStatus <> SpeechRecognitionUIStatus.Succeeded Then
                    ' Return if not successful. This is a shortcut. A real application would take appropriate action depending 
                    ' on the status
                    Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("An error occurred in speech recognition."))
                    Exit Sub
                End If

                ' Recognition successful -- take the action specified by the user
                Select Case result.RecognitionResult.Text.ToLower
                    ' Save this memo and get another one
                    Case "save and try again"

                        Memo_TB.Dispatcher.BeginInvoke(Sub() Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text))

                        Utilities.SetTextBoxTextSafely(Memo_TB, "")

                        numMemosSaved = numMemosSaved + 1

                        GetNewMemoByVoice()

                        ' Save memo and go back to screen
                    Case "save"

                        Memo_TB.Dispatcher.BeginInvoke(Sub() Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text))

                        numMemosSaved = numMemosSaved + 1

                        Utilities.SetTextBoxTextSafely(Memo_TB, "")

                        ' Erase this memo and try again
                    Case "try again"

                        Utilities.SetTextBoxTextSafely(Memo_TB, "")

                        GetNewMemoByVoice()

                        ' Erase this memo and go back to screen
                    Case "cancel"

                        Utilities.SetTextBoxTextSafely(Memo_TB, "")

                        ' Save and go to view memos page
                    Case "view saved memos"

                        Memo_TB.Dispatcher.BeginInvoke(Sub() Utilities.WriteLineToIsoStorage(fileName, Memo_TB.Text))

                        numMemosSaved = numMemosSaved + 1

                        Utilities.SetTextBoxTextSafely(Memo_TB, "")

                        Try
                            Dispatcher.BeginInvoke(Sub() NavigationService.Navigate(viewMemosUri))
                        Catch ex As Exception
                            Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Exception in navigating to 'view memos' page" & ex.Message))
                        End Try

                        ' Leave this memo and go back to screen
                    Case "quit"

                    Case Else
                End Select
            Catch ex As Exception
                Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Exception in command recognition handler " & ex.Message))
            End Try
        End Sub

		' Updates the "num" PhraseList to have the same number of entries as the number of saved memos.
		' This supports "Magic Memo show memo 5" if there are five or more memos saved.
		Private async Sub UpdateNumberPhraseList(ByVal phraseList As String, ByVal newLimit As Integer, ByVal voiceSetName As String)
			' Helper function that sets string array to {"1", "2", etc}
			Dim positiveIntegers As List(Of String) = Utilities.GetStringListOfPositiveIntegers(Math.Max(1, newLimit))

			Try
				Dim vcs As VoiceCommandSet = Nothing

				If VoiceCommandService.InstalledCommandSets.TryGetValue(voiceSetName, vcs) Then
					' Update phrase list to the new numbers
					await vcs.UpdatePhraseListAsync(phraseList, positiveIntegers)
				End If
			Catch ex As Exception
				Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Exception in UpdateNumberPhraseList " & ex.Message))
			End Try
		End Sub

		' Recursively crawls through the tree of controls to find all button controls and adds the Content text
		' to a List.
		Public Shared Sub GetButtonContentText(ByVal uiElement As UIElement, ByVal buttonNames As IList(Of String))
			Dim button As Button = TryCast(uiElement, Button)
			If button IsNot Nothing AndAlso button.Content IsNot Nothing AndAlso (Not String.IsNullOrEmpty(button.Content.ToString)) Then
				buttonNames.Add(button.Content.ToString) ' TODO: check to make sure the Content is not a non-text object
			End If

			Dim panel As Panel = TryCast(uiElement, Panel)

			If panel IsNot Nothing Then
				For Each child As UIElement In panel.Children
					GetButtonContentText(child, buttonNames)
				Next child
			End If
		End Sub
		#End Region

	End Class
End Namespace

